# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ... import core
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.add_project_response_model import AddProjectResponseModel
from ...types.convert_project_response_model import ConvertProjectResponseModel
from ...types.delete_project_response_model import DeleteProjectResponseModel
from ...types.edit_project_response_model import EditProjectResponseModel
from ...types.get_projects_response import GetProjectsResponse
from ...types.project_extended_response import ProjectExtendedResponse
from ...types.project_muted_tracks_response_model import ProjectMutedTracksResponseModel
from ...types.quality_preset_type import QualityPresetType
from .raw_client import AsyncRawProjectsClient, RawProjectsClient
from .types.projects_create_request_apply_text_normalization import ProjectsCreateRequestApplyTextNormalization
from .types.projects_create_request_fiction import ProjectsCreateRequestFiction
from .types.projects_create_request_source_type import ProjectsCreateRequestSourceType
from .types.projects_create_request_target_audience import ProjectsCreateRequestTargetAudience

if typing.TYPE_CHECKING:
    from .chapters.client import AsyncChaptersClient, ChaptersClient
    from .content.client import AsyncContentClient, ContentClient
    from .pronunciation_dictionaries.client import AsyncPronunciationDictionariesClient, PronunciationDictionariesClient
    from .snapshots.client import AsyncSnapshotsClient, SnapshotsClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ProjectsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawProjectsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._pronunciation_dictionaries: typing.Optional[PronunciationDictionariesClient] = None
        self._content: typing.Optional[ContentClient] = None
        self._snapshots: typing.Optional[SnapshotsClient] = None
        self._chapters: typing.Optional[ChaptersClient] = None

    @property
    def with_raw_response(self) -> RawProjectsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawProjectsClient
        """
        return self._raw_client

    def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> GetProjectsResponse:
        """
        Returns a list of your Studio projects with metadata.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetProjectsResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.studio.projects.list()
        """
        _response = self._raw_client.list(request_options=request_options)
        return _response.data

    def create(
        self,
        *,
        name: str,
        default_title_voice_id: typing.Optional[str] = OMIT,
        default_paragraph_voice_id: typing.Optional[str] = OMIT,
        default_model_id: typing.Optional[str] = OMIT,
        from_url: typing.Optional[str] = OMIT,
        from_document: typing.Optional[core.File] = OMIT,
        from_content_json: typing.Optional[str] = OMIT,
        quality_preset: typing.Optional[QualityPresetType] = OMIT,
        title: typing.Optional[str] = OMIT,
        author: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        genres: typing.Optional[typing.List[str]] = OMIT,
        target_audience: typing.Optional[ProjectsCreateRequestTargetAudience] = OMIT,
        language: typing.Optional[str] = OMIT,
        content_type: typing.Optional[str] = OMIT,
        original_publication_date: typing.Optional[str] = OMIT,
        mature_content: typing.Optional[bool] = OMIT,
        isbn_number: typing.Optional[str] = OMIT,
        acx_volume_normalization: typing.Optional[bool] = OMIT,
        volume_normalization: typing.Optional[bool] = OMIT,
        pronunciation_dictionary_locators: typing.Optional[typing.List[str]] = OMIT,
        callback_url: typing.Optional[str] = OMIT,
        fiction: typing.Optional[ProjectsCreateRequestFiction] = OMIT,
        apply_text_normalization: typing.Optional[ProjectsCreateRequestApplyTextNormalization] = OMIT,
        auto_convert: typing.Optional[bool] = OMIT,
        auto_assign_voices: typing.Optional[bool] = OMIT,
        source_type: typing.Optional[ProjectsCreateRequestSourceType] = OMIT,
        voice_settings: typing.Optional[typing.List[str]] = OMIT,
        create_publishing_read: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddProjectResponseModel:
        """
        Creates a new Studio project, it can be either initialized as blank, from a document or from a URL.

        Parameters
        ----------
        name : str
            The name of the Studio project, used for identification only.

        default_title_voice_id : typing.Optional[str]
            The voice_id that corresponds to the default voice used for new titles.

        default_paragraph_voice_id : typing.Optional[str]
            The voice_id that corresponds to the default voice used for new paragraphs.

        default_model_id : typing.Optional[str]
            The ID of the model to be used for this Studio project, you can query GET /v1/models to list all available models.

        from_url : typing.Optional[str]
            An optional URL from which we will extract content to initialize the Studio project. If this is set, 'from_url' and 'from_content' must be null. If neither 'from_url', 'from_document', 'from_content' are provided we will initialize the Studio project as blank.

        from_document : typing.Optional[core.File]
            See core.File for more documentation

        from_content_json : typing.Optional[str]

                An optional content to initialize the Studio project with. If this is set, 'from_url' and 'from_document' must be null. If neither 'from_url', 'from_document', 'from_content' are provided we will initialize the Studio project as blank.

                Example:
                [{"name": "Chapter A", "blocks": [{"sub_type": "p", "nodes": [{"voice_id": "6lCwbsX1yVjD49QmpkT0", "text": "A", "type": "tts_node"}, {"voice_id": "6lCwbsX1yVjD49QmpkT1", "text": "B", "type": "tts_node"}]}, {"sub_type": "h1", "nodes": [{"voice_id": "6lCwbsX1yVjD49QmpkT0", "text": "C", "type": "tts_node"}, {"voice_id": "6lCwbsX1yVjD49QmpkT1", "text": "D", "type": "tts_node"}]}]}, {"name": "Chapter B", "blocks": [{"sub_type": "p", "nodes": [{"voice_id": "6lCwbsX1yVjD49QmpkT0", "text": "E", "type": "tts_node"}, {"voice_id": "6lCwbsX1yVjD49QmpkT1", "text": "F", "type": "tts_node"}]}, {"sub_type": "h2", "nodes": [{"voice_id": "6lCwbsX1yVjD49QmpkT0", "text": "G", "type": "tts_node"}, {"voice_id": "6lCwbsX1yVjD49QmpkT1", "text": "H", "type": "tts_node"}]}]}]


        quality_preset : typing.Optional[QualityPresetType]
            Output quality of the generated audio. Must be one of:
            'standard' - standard output format, 128kbps with 44.1kHz sample rate.
            'high' - high quality output format, 192kbps with 44.1kHz sample rate and major improvements on our side.
            'ultra' - ultra quality output format, 192kbps with 44.1kHz sample rate and highest improvements on our side.
            'ultra_lossless' - ultra quality output format, 705.6kbps with 44.1kHz sample rate and highest improvements on our side in a fully lossless format.

        title : typing.Optional[str]
            An optional name of the author of the Studio project, this will be added as metadata to the mp3 file on Studio project or chapter download.

        author : typing.Optional[str]
            An optional name of the author of the Studio project, this will be added as metadata to the mp3 file on Studio project or chapter download.

        description : typing.Optional[str]
            An optional description of the Studio project.

        genres : typing.Optional[typing.List[str]]
            An optional list of genres associated with the Studio project.

        target_audience : typing.Optional[ProjectsCreateRequestTargetAudience]
            An optional target audience of the Studio project.

        language : typing.Optional[str]
            An optional language of the Studio project. Two-letter language code (ISO 639-1).

        content_type : typing.Optional[str]
            An optional content type of the Studio project.

        original_publication_date : typing.Optional[str]
            An optional original publication date of the Studio project, in the format YYYY-MM-DD or YYYY.

        mature_content : typing.Optional[bool]
            An optional specification of whether this Studio project contains mature content.

        isbn_number : typing.Optional[str]
            An optional ISBN number of the Studio project you want to create, this will be added as metadata to the mp3 file on Studio project or chapter download.

        acx_volume_normalization : typing.Optional[bool]
            [Deprecated] When the Studio project is downloaded, should the returned audio have postprocessing in order to make it compliant with audiobook normalized volume requirements

        volume_normalization : typing.Optional[bool]
            When the Studio project is downloaded, should the returned audio have postprocessing in order to make it compliant with audiobook normalized volume requirements

        pronunciation_dictionary_locators : typing.Optional[typing.List[str]]
            A list of pronunciation dictionary locators (pronunciation_dictionary_id, version_id) encoded as a list of JSON strings for pronunciation dictionaries to be applied to the text. A list of json encoded strings is required as adding projects may occur through formData as opposed to jsonBody. To specify multiple dictionaries use multiple --form lines in your curl, such as --form 'pronunciation_dictionary_locators="{\\"pronunciation_dictionary_id\\":\\"Vmd4Zor6fplcA7WrINey\\",\\"version_id\\":\\"hRPaxjlTdR7wFMhV4w0b\\"}"' --form 'pronunciation_dictionary_locators="{\\"pronunciation_dictionary_id\\":\\"JzWtcGQMJ6bnlWwyMo7e\\",\\"version_id\\":\\"lbmwxiLu4q6txYxgdZqn\\"}"'.

        callback_url : typing.Optional[str]

                A url that will be called by our service when the Studio project is converted. Request will contain a json blob containing the status of the conversion
                Messages:
                1. When project was converted successfully:
                {
                  type: "project_conversion_status",
                  event_timestamp: 1234567890,
                  data: {
                    request_id: "1234567890",
                    project_id: "21m00Tcm4TlvDq8ikWAM",
                    conversion_status: "success",
                    project_snapshot_id: "22m00Tcm4TlvDq8ikMAT",
                    error_details: None,
                  }
                }
                2. When project conversion failed:
                {
                  type: "project_conversion_status",
                  event_timestamp: 1234567890,
                  data: {
                    request_id: "1234567890",
                    project_id: "21m00Tcm4TlvDq8ikWAM",
                    conversion_status: "error",
                    project_snapshot_id: None,
                    error_details: "Error details if conversion failed"
                  }
                }

                3. When chapter was converted successfully:
                {
                  type: "chapter_conversion_status",
                  event_timestamp: 1234567890,
                  data: {
                    request_id: "1234567890",
                    project_id: "21m00Tcm4TlvDq8ikWAM",
                    chapter_id: "22m00Tcm4TlvDq8ikMAT",
                    conversion_status: "success",
                    chapter_snapshot_id: "23m00Tcm4TlvDq8ikMAV",
                    error_details: None,
                  }
                }
                4. When chapter conversion failed:
                {
                  type: "chapter_conversion_status",
                  event_timestamp: 1234567890,
                  data: {
                    request_id: "1234567890",
                    project_id: "21m00Tcm4TlvDq8ikWAM",
                    chapter_id: "22m00Tcm4TlvDq8ikMAT",
                    conversion_status: "error",
                    chapter_snapshot_id: None,
                    error_details: "Error details if conversion failed"
                  }
                }


        fiction : typing.Optional[ProjectsCreateRequestFiction]
            An optional specification of whether the content of this Studio project is fiction.

        apply_text_normalization : typing.Optional[ProjectsCreateRequestApplyTextNormalization]

                This parameter controls text normalization with four modes: 'auto', 'on', 'apply_english' and 'off'.
                When set to 'auto', the system will automatically decide whether to apply text normalization
                (e.g., spelling out numbers). With 'on', text normalization will always be applied, while
                with 'off', it will be skipped. 'apply_english' is the same as 'on' but will assume that text is in English.


        auto_convert : typing.Optional[bool]
            Whether to auto convert the Studio project to audio or not.

        auto_assign_voices : typing.Optional[bool]
            [Alpha Feature] Whether automatically assign voices to phrases in the create Project.

        source_type : typing.Optional[ProjectsCreateRequestSourceType]
            The type of Studio project to create.

        voice_settings : typing.Optional[typing.List[str]]
                Optional voice settings overrides for the project, encoded as a list of JSON strings.

                Example:
                ["{\\"voice_id\\": \\"21m00Tcm4TlvDq8ikWAM\\", \\"stability\\": 0.7, \\"similarity_boost\\": 0.8, \\"style\\": 0.5, \\"speed\\": 1.0, \\"use_speaker_boost\\": true}"]


        create_publishing_read : typing.Optional[bool]
            If true, creates a corresponding read for direct publishing in draft state

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddProjectResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.studio.projects.create(
            name="name",
        )
        """
        _response = self._raw_client.create(
            name=name,
            default_title_voice_id=default_title_voice_id,
            default_paragraph_voice_id=default_paragraph_voice_id,
            default_model_id=default_model_id,
            from_url=from_url,
            from_document=from_document,
            from_content_json=from_content_json,
            quality_preset=quality_preset,
            title=title,
            author=author,
            description=description,
            genres=genres,
            target_audience=target_audience,
            language=language,
            content_type=content_type,
            original_publication_date=original_publication_date,
            mature_content=mature_content,
            isbn_number=isbn_number,
            acx_volume_normalization=acx_volume_normalization,
            volume_normalization=volume_normalization,
            pronunciation_dictionary_locators=pronunciation_dictionary_locators,
            callback_url=callback_url,
            fiction=fiction,
            apply_text_normalization=apply_text_normalization,
            auto_convert=auto_convert,
            auto_assign_voices=auto_assign_voices,
            source_type=source_type,
            voice_settings=voice_settings,
            create_publishing_read=create_publishing_read,
            request_options=request_options,
        )
        return _response.data

    def get(
        self,
        project_id: str,
        *,
        share_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProjectExtendedResponse:
        """
        Returns information about a specific Studio project. This endpoint returns more detailed information about a project than `GET /v1/studio`.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        share_id : typing.Optional[str]
            The share ID of the project

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectExtendedResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.studio.projects.get(
            project_id="21m00Tcm4TlvDq8ikWAM",
            share_id="share_id",
        )
        """
        _response = self._raw_client.get(project_id, share_id=share_id, request_options=request_options)
        return _response.data

    def update(
        self,
        project_id: str,
        *,
        name: str,
        default_title_voice_id: str,
        default_paragraph_voice_id: str,
        title: typing.Optional[str] = OMIT,
        author: typing.Optional[str] = OMIT,
        isbn_number: typing.Optional[str] = OMIT,
        volume_normalization: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EditProjectResponseModel:
        """
        Updates the specified Studio project by setting the values of the parameters passed.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        name : str
            The name of the Studio project, used for identification only.

        default_title_voice_id : str
            The voice_id that corresponds to the default voice used for new titles.

        default_paragraph_voice_id : str
            The voice_id that corresponds to the default voice used for new paragraphs.

        title : typing.Optional[str]
            An optional name of the author of the Studio project, this will be added as metadata to the mp3 file on Studio project or chapter download.

        author : typing.Optional[str]
            An optional name of the author of the Studio project, this will be added as metadata to the mp3 file on Studio project or chapter download.

        isbn_number : typing.Optional[str]
            An optional ISBN number of the Studio project you want to create, this will be added as metadata to the mp3 file on Studio project or chapter download.

        volume_normalization : typing.Optional[bool]
            When the Studio project is downloaded, should the returned audio have postprocessing in order to make it compliant with audiobook normalized volume requirements

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EditProjectResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.studio.projects.update(
            project_id="21m00Tcm4TlvDq8ikWAM",
            name="Project 1",
            default_title_voice_id="21m00Tcm4TlvDq8ikWAM",
            default_paragraph_voice_id="21m00Tcm4TlvDq8ikWAM",
        )
        """
        _response = self._raw_client.update(
            project_id,
            name=name,
            default_title_voice_id=default_title_voice_id,
            default_paragraph_voice_id=default_paragraph_voice_id,
            title=title,
            author=author,
            isbn_number=isbn_number,
            volume_normalization=volume_normalization,
            request_options=request_options,
        )
        return _response.data

    def delete(
        self, project_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteProjectResponseModel:
        """
        Deletes a Studio project.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteProjectResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.studio.projects.delete(
            project_id="21m00Tcm4TlvDq8ikWAM",
        )
        """
        _response = self._raw_client.delete(project_id, request_options=request_options)
        return _response.data

    def convert(
        self, project_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ConvertProjectResponseModel:
        """
        Starts conversion of a Studio project and all of its chapters.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConvertProjectResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.studio.projects.convert(
            project_id="21m00Tcm4TlvDq8ikWAM",
        )
        """
        _response = self._raw_client.convert(project_id, request_options=request_options)
        return _response.data

    def get_muted_tracks(
        self, project_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ProjectMutedTracksResponseModel:
        """
        Returns a list of chapter IDs that have muted tracks in a project.

        Parameters
        ----------
        project_id : str
            The ID of the Studio project.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectMutedTracksResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.studio.projects.get_muted_tracks(
            project_id="21m00Tcm4TlvDq8ikWAM",
        )
        """
        _response = self._raw_client.get_muted_tracks(project_id, request_options=request_options)
        return _response.data

    @property
    def pronunciation_dictionaries(self):
        if self._pronunciation_dictionaries is None:
            from .pronunciation_dictionaries.client import PronunciationDictionariesClient  # noqa: E402

            self._pronunciation_dictionaries = PronunciationDictionariesClient(client_wrapper=self._client_wrapper)
        return self._pronunciation_dictionaries

    @property
    def content(self):
        if self._content is None:
            from .content.client import ContentClient  # noqa: E402

            self._content = ContentClient(client_wrapper=self._client_wrapper)
        return self._content

    @property
    def snapshots(self):
        if self._snapshots is None:
            from .snapshots.client import SnapshotsClient  # noqa: E402

            self._snapshots = SnapshotsClient(client_wrapper=self._client_wrapper)
        return self._snapshots

    @property
    def chapters(self):
        if self._chapters is None:
            from .chapters.client import ChaptersClient  # noqa: E402

            self._chapters = ChaptersClient(client_wrapper=self._client_wrapper)
        return self._chapters


class AsyncProjectsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawProjectsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._pronunciation_dictionaries: typing.Optional[AsyncPronunciationDictionariesClient] = None
        self._content: typing.Optional[AsyncContentClient] = None
        self._snapshots: typing.Optional[AsyncSnapshotsClient] = None
        self._chapters: typing.Optional[AsyncChaptersClient] = None

    @property
    def with_raw_response(self) -> AsyncRawProjectsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawProjectsClient
        """
        return self._raw_client

    async def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> GetProjectsResponse:
        """
        Returns a list of your Studio projects with metadata.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetProjectsResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.studio.projects.list()


        asyncio.run(main())
        """
        _response = await self._raw_client.list(request_options=request_options)
        return _response.data

    async def create(
        self,
        *,
        name: str,
        default_title_voice_id: typing.Optional[str] = OMIT,
        default_paragraph_voice_id: typing.Optional[str] = OMIT,
        default_model_id: typing.Optional[str] = OMIT,
        from_url: typing.Optional[str] = OMIT,
        from_document: typing.Optional[core.File] = OMIT,
        from_content_json: typing.Optional[str] = OMIT,
        quality_preset: typing.Optional[QualityPresetType] = OMIT,
        title: typing.Optional[str] = OMIT,
        author: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        genres: typing.Optional[typing.List[str]] = OMIT,
        target_audience: typing.Optional[ProjectsCreateRequestTargetAudience] = OMIT,
        language: typing.Optional[str] = OMIT,
        content_type: typing.Optional[str] = OMIT,
        original_publication_date: typing.Optional[str] = OMIT,
        mature_content: typing.Optional[bool] = OMIT,
        isbn_number: typing.Optional[str] = OMIT,
        acx_volume_normalization: typing.Optional[bool] = OMIT,
        volume_normalization: typing.Optional[bool] = OMIT,
        pronunciation_dictionary_locators: typing.Optional[typing.List[str]] = OMIT,
        callback_url: typing.Optional[str] = OMIT,
        fiction: typing.Optional[ProjectsCreateRequestFiction] = OMIT,
        apply_text_normalization: typing.Optional[ProjectsCreateRequestApplyTextNormalization] = OMIT,
        auto_convert: typing.Optional[bool] = OMIT,
        auto_assign_voices: typing.Optional[bool] = OMIT,
        source_type: typing.Optional[ProjectsCreateRequestSourceType] = OMIT,
        voice_settings: typing.Optional[typing.List[str]] = OMIT,
        create_publishing_read: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddProjectResponseModel:
        """
        Creates a new Studio project, it can be either initialized as blank, from a document or from a URL.

        Parameters
        ----------
        name : str
            The name of the Studio project, used for identification only.

        default_title_voice_id : typing.Optional[str]
            The voice_id that corresponds to the default voice used for new titles.

        default_paragraph_voice_id : typing.Optional[str]
            The voice_id that corresponds to the default voice used for new paragraphs.

        default_model_id : typing.Optional[str]
            The ID of the model to be used for this Studio project, you can query GET /v1/models to list all available models.

        from_url : typing.Optional[str]
            An optional URL from which we will extract content to initialize the Studio project. If this is set, 'from_url' and 'from_content' must be null. If neither 'from_url', 'from_document', 'from_content' are provided we will initialize the Studio project as blank.

        from_document : typing.Optional[core.File]
            See core.File for more documentation

        from_content_json : typing.Optional[str]

                An optional content to initialize the Studio project with. If this is set, 'from_url' and 'from_document' must be null. If neither 'from_url', 'from_document', 'from_content' are provided we will initialize the Studio project as blank.

                Example:
                [{"name": "Chapter A", "blocks": [{"sub_type": "p", "nodes": [{"voice_id": "6lCwbsX1yVjD49QmpkT0", "text": "A", "type": "tts_node"}, {"voice_id": "6lCwbsX1yVjD49QmpkT1", "text": "B", "type": "tts_node"}]}, {"sub_type": "h1", "nodes": [{"voice_id": "6lCwbsX1yVjD49QmpkT0", "text": "C", "type": "tts_node"}, {"voice_id": "6lCwbsX1yVjD49QmpkT1", "text": "D", "type": "tts_node"}]}]}, {"name": "Chapter B", "blocks": [{"sub_type": "p", "nodes": [{"voice_id": "6lCwbsX1yVjD49QmpkT0", "text": "E", "type": "tts_node"}, {"voice_id": "6lCwbsX1yVjD49QmpkT1", "text": "F", "type": "tts_node"}]}, {"sub_type": "h2", "nodes": [{"voice_id": "6lCwbsX1yVjD49QmpkT0", "text": "G", "type": "tts_node"}, {"voice_id": "6lCwbsX1yVjD49QmpkT1", "text": "H", "type": "tts_node"}]}]}]


        quality_preset : typing.Optional[QualityPresetType]
            Output quality of the generated audio. Must be one of:
            'standard' - standard output format, 128kbps with 44.1kHz sample rate.
            'high' - high quality output format, 192kbps with 44.1kHz sample rate and major improvements on our side.
            'ultra' - ultra quality output format, 192kbps with 44.1kHz sample rate and highest improvements on our side.
            'ultra_lossless' - ultra quality output format, 705.6kbps with 44.1kHz sample rate and highest improvements on our side in a fully lossless format.

        title : typing.Optional[str]
            An optional name of the author of the Studio project, this will be added as metadata to the mp3 file on Studio project or chapter download.

        author : typing.Optional[str]
            An optional name of the author of the Studio project, this will be added as metadata to the mp3 file on Studio project or chapter download.

        description : typing.Optional[str]
            An optional description of the Studio project.

        genres : typing.Optional[typing.List[str]]
            An optional list of genres associated with the Studio project.

        target_audience : typing.Optional[ProjectsCreateRequestTargetAudience]
            An optional target audience of the Studio project.

        language : typing.Optional[str]
            An optional language of the Studio project. Two-letter language code (ISO 639-1).

        content_type : typing.Optional[str]
            An optional content type of the Studio project.

        original_publication_date : typing.Optional[str]
            An optional original publication date of the Studio project, in the format YYYY-MM-DD or YYYY.

        mature_content : typing.Optional[bool]
            An optional specification of whether this Studio project contains mature content.

        isbn_number : typing.Optional[str]
            An optional ISBN number of the Studio project you want to create, this will be added as metadata to the mp3 file on Studio project or chapter download.

        acx_volume_normalization : typing.Optional[bool]
            [Deprecated] When the Studio project is downloaded, should the returned audio have postprocessing in order to make it compliant with audiobook normalized volume requirements

        volume_normalization : typing.Optional[bool]
            When the Studio project is downloaded, should the returned audio have postprocessing in order to make it compliant with audiobook normalized volume requirements

        pronunciation_dictionary_locators : typing.Optional[typing.List[str]]
            A list of pronunciation dictionary locators (pronunciation_dictionary_id, version_id) encoded as a list of JSON strings for pronunciation dictionaries to be applied to the text. A list of json encoded strings is required as adding projects may occur through formData as opposed to jsonBody. To specify multiple dictionaries use multiple --form lines in your curl, such as --form 'pronunciation_dictionary_locators="{\\"pronunciation_dictionary_id\\":\\"Vmd4Zor6fplcA7WrINey\\",\\"version_id\\":\\"hRPaxjlTdR7wFMhV4w0b\\"}"' --form 'pronunciation_dictionary_locators="{\\"pronunciation_dictionary_id\\":\\"JzWtcGQMJ6bnlWwyMo7e\\",\\"version_id\\":\\"lbmwxiLu4q6txYxgdZqn\\"}"'.

        callback_url : typing.Optional[str]

                A url that will be called by our service when the Studio project is converted. Request will contain a json blob containing the status of the conversion
                Messages:
                1. When project was converted successfully:
                {
                  type: "project_conversion_status",
                  event_timestamp: 1234567890,
                  data: {
                    request_id: "1234567890",
                    project_id: "21m00Tcm4TlvDq8ikWAM",
                    conversion_status: "success",
                    project_snapshot_id: "22m00Tcm4TlvDq8ikMAT",
                    error_details: None,
                  }
                }
                2. When project conversion failed:
                {
                  type: "project_conversion_status",
                  event_timestamp: 1234567890,
                  data: {
                    request_id: "1234567890",
                    project_id: "21m00Tcm4TlvDq8ikWAM",
                    conversion_status: "error",
                    project_snapshot_id: None,
                    error_details: "Error details if conversion failed"
                  }
                }

                3. When chapter was converted successfully:
                {
                  type: "chapter_conversion_status",
                  event_timestamp: 1234567890,
                  data: {
                    request_id: "1234567890",
                    project_id: "21m00Tcm4TlvDq8ikWAM",
                    chapter_id: "22m00Tcm4TlvDq8ikMAT",
                    conversion_status: "success",
                    chapter_snapshot_id: "23m00Tcm4TlvDq8ikMAV",
                    error_details: None,
                  }
                }
                4. When chapter conversion failed:
                {
                  type: "chapter_conversion_status",
                  event_timestamp: 1234567890,
                  data: {
                    request_id: "1234567890",
                    project_id: "21m00Tcm4TlvDq8ikWAM",
                    chapter_id: "22m00Tcm4TlvDq8ikMAT",
                    conversion_status: "error",
                    chapter_snapshot_id: None,
                    error_details: "Error details if conversion failed"
                  }
                }


        fiction : typing.Optional[ProjectsCreateRequestFiction]
            An optional specification of whether the content of this Studio project is fiction.

        apply_text_normalization : typing.Optional[ProjectsCreateRequestApplyTextNormalization]

                This parameter controls text normalization with four modes: 'auto', 'on', 'apply_english' and 'off'.
                When set to 'auto', the system will automatically decide whether to apply text normalization
                (e.g., spelling out numbers). With 'on', text normalization will always be applied, while
                with 'off', it will be skipped. 'apply_english' is the same as 'on' but will assume that text is in English.


        auto_convert : typing.Optional[bool]
            Whether to auto convert the Studio project to audio or not.

        auto_assign_voices : typing.Optional[bool]
            [Alpha Feature] Whether automatically assign voices to phrases in the create Project.

        source_type : typing.Optional[ProjectsCreateRequestSourceType]
            The type of Studio project to create.

        voice_settings : typing.Optional[typing.List[str]]
                Optional voice settings overrides for the project, encoded as a list of JSON strings.

                Example:
                ["{\\"voice_id\\": \\"21m00Tcm4TlvDq8ikWAM\\", \\"stability\\": 0.7, \\"similarity_boost\\": 0.8, \\"style\\": 0.5, \\"speed\\": 1.0, \\"use_speaker_boost\\": true}"]


        create_publishing_read : typing.Optional[bool]
            If true, creates a corresponding read for direct publishing in draft state

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddProjectResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.studio.projects.create(
                name="name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            name=name,
            default_title_voice_id=default_title_voice_id,
            default_paragraph_voice_id=default_paragraph_voice_id,
            default_model_id=default_model_id,
            from_url=from_url,
            from_document=from_document,
            from_content_json=from_content_json,
            quality_preset=quality_preset,
            title=title,
            author=author,
            description=description,
            genres=genres,
            target_audience=target_audience,
            language=language,
            content_type=content_type,
            original_publication_date=original_publication_date,
            mature_content=mature_content,
            isbn_number=isbn_number,
            acx_volume_normalization=acx_volume_normalization,
            volume_normalization=volume_normalization,
            pronunciation_dictionary_locators=pronunciation_dictionary_locators,
            callback_url=callback_url,
            fiction=fiction,
            apply_text_normalization=apply_text_normalization,
            auto_convert=auto_convert,
            auto_assign_voices=auto_assign_voices,
            source_type=source_type,
            voice_settings=voice_settings,
            create_publishing_read=create_publishing_read,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self,
        project_id: str,
        *,
        share_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProjectExtendedResponse:
        """
        Returns information about a specific Studio project. This endpoint returns more detailed information about a project than `GET /v1/studio`.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        share_id : typing.Optional[str]
            The share ID of the project

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectExtendedResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.studio.projects.get(
                project_id="21m00Tcm4TlvDq8ikWAM",
                share_id="share_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(project_id, share_id=share_id, request_options=request_options)
        return _response.data

    async def update(
        self,
        project_id: str,
        *,
        name: str,
        default_title_voice_id: str,
        default_paragraph_voice_id: str,
        title: typing.Optional[str] = OMIT,
        author: typing.Optional[str] = OMIT,
        isbn_number: typing.Optional[str] = OMIT,
        volume_normalization: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EditProjectResponseModel:
        """
        Updates the specified Studio project by setting the values of the parameters passed.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        name : str
            The name of the Studio project, used for identification only.

        default_title_voice_id : str
            The voice_id that corresponds to the default voice used for new titles.

        default_paragraph_voice_id : str
            The voice_id that corresponds to the default voice used for new paragraphs.

        title : typing.Optional[str]
            An optional name of the author of the Studio project, this will be added as metadata to the mp3 file on Studio project or chapter download.

        author : typing.Optional[str]
            An optional name of the author of the Studio project, this will be added as metadata to the mp3 file on Studio project or chapter download.

        isbn_number : typing.Optional[str]
            An optional ISBN number of the Studio project you want to create, this will be added as metadata to the mp3 file on Studio project or chapter download.

        volume_normalization : typing.Optional[bool]
            When the Studio project is downloaded, should the returned audio have postprocessing in order to make it compliant with audiobook normalized volume requirements

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EditProjectResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.studio.projects.update(
                project_id="21m00Tcm4TlvDq8ikWAM",
                name="Project 1",
                default_title_voice_id="21m00Tcm4TlvDq8ikWAM",
                default_paragraph_voice_id="21m00Tcm4TlvDq8ikWAM",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            project_id,
            name=name,
            default_title_voice_id=default_title_voice_id,
            default_paragraph_voice_id=default_paragraph_voice_id,
            title=title,
            author=author,
            isbn_number=isbn_number,
            volume_normalization=volume_normalization,
            request_options=request_options,
        )
        return _response.data

    async def delete(
        self, project_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteProjectResponseModel:
        """
        Deletes a Studio project.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteProjectResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.studio.projects.delete(
                project_id="21m00Tcm4TlvDq8ikWAM",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(project_id, request_options=request_options)
        return _response.data

    async def convert(
        self, project_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ConvertProjectResponseModel:
        """
        Starts conversion of a Studio project and all of its chapters.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConvertProjectResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.studio.projects.convert(
                project_id="21m00Tcm4TlvDq8ikWAM",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.convert(project_id, request_options=request_options)
        return _response.data

    async def get_muted_tracks(
        self, project_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ProjectMutedTracksResponseModel:
        """
        Returns a list of chapter IDs that have muted tracks in a project.

        Parameters
        ----------
        project_id : str
            The ID of the Studio project.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectMutedTracksResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.studio.projects.get_muted_tracks(
                project_id="21m00Tcm4TlvDq8ikWAM",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_muted_tracks(project_id, request_options=request_options)
        return _response.data

    @property
    def pronunciation_dictionaries(self):
        if self._pronunciation_dictionaries is None:
            from .pronunciation_dictionaries.client import AsyncPronunciationDictionariesClient  # noqa: E402

            self._pronunciation_dictionaries = AsyncPronunciationDictionariesClient(client_wrapper=self._client_wrapper)
        return self._pronunciation_dictionaries

    @property
    def content(self):
        if self._content is None:
            from .content.client import AsyncContentClient  # noqa: E402

            self._content = AsyncContentClient(client_wrapper=self._client_wrapper)
        return self._content

    @property
    def snapshots(self):
        if self._snapshots is None:
            from .snapshots.client import AsyncSnapshotsClient  # noqa: E402

            self._snapshots = AsyncSnapshotsClient(client_wrapper=self._client_wrapper)
        return self._snapshots

    @property
    def chapters(self):
        if self._chapters is None:
            from .chapters.client import AsyncChaptersClient  # noqa: E402

            self._chapters = AsyncChaptersClient(client_wrapper=self._client_wrapper)
        return self._chapters



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel


class RealtimeVoiceSettings(UncheckedBaseModel):
    stability: typing.Optional[float] = pydantic.Field(default=None)
    """
    Defines the stability for voice settings.
    """

    similarity_boost: typing.Optional[float] = pydantic.Field(default=None)
    """
    Defines the similarity boost for voice settings.
    """

    style: typing.Optional[float] = pydantic.Field(default=None)
    """
    Defines the style for voice settings. This parameter is available on V2+ models.
    """

    use_speaker_boost: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Defines the use speaker boost for voice settings. This parameter is available on V2+ models.
    """

    speed: typing.Optional[float] = pydantic.Field(default=None)
    """
    Controls the speed of the generated speech. Values range from 0.7 to 1.2, with 1.0 being the default speed.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .dialogue_text_alignment import DialogueTextAlignment


class TextToDialogueWebsocketAudioChunk(UncheckedBaseModel):
    """
    Server chunk containing encoded audio and optional alignment metadata.
    """

    audio: typing.Optional[str] = pydantic.Field(default=None)
    """
    Base64-encoded audio bytes for the selected `output_format`.
    """

    alignment: typing.Optional[DialogueTextAlignment] = pydantic.Field(default=None)
    """
    Present when `sync_alignment` query parameter is `true` and the model returned timing data for the chunk.
    """

    normalized_alignment: typing.Optional[DialogueTextAlignment] = pydantic.Field(default=None)
    """
    Reserved for future use; currently unused by the server.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ....core.api_error import ApiError
from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.http_response import AsyncHttpResponse, HttpResponse
from ....core.jsonable_encoder import jsonable_encoder
from ....core.request_options import RequestOptions
from ....core.serialization import convert_and_respect_annotation_metadata
from ....core.unchecked_base_model import construct_type
from ....errors.unprocessable_entity_error import UnprocessableEntityError
from ....types.add_chapter_response_model import AddChapterResponseModel
from ....types.chapter_content_input_model import ChapterContentInputModel
from ....types.chapter_with_content_response_model import ChapterWithContentResponseModel
from ....types.convert_chapter_response_model import ConvertChapterResponseModel
from ....types.delete_chapter_response_model import DeleteChapterResponseModel
from ....types.edit_chapter_response_model import EditChapterResponseModel
from ....types.get_chapters_response import GetChaptersResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawChaptersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self, project_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetChaptersResponse]:
        """
        Returns a list of a Studio project's chapters.

        Parameters
        ----------
        project_id : str
            The ID of the Studio project.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetChaptersResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/studio/projects/{jsonable_encoder(project_id)}/chapters",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetChaptersResponse,
                    construct_type(
                        type_=GetChaptersResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        project_id: str,
        *,
        name: str,
        from_url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AddChapterResponseModel]:
        """
        Creates a new chapter either as blank or from a URL.

        Parameters
        ----------
        project_id : str
            The ID of the Studio project.

        name : str
            The name of the chapter, used for identification only.

        from_url : typing.Optional[str]
            An optional URL from which we will extract content to initialize the Studio project. If this is set, 'from_url' and 'from_content' must be null. If neither 'from_url', 'from_document', 'from_content' are provided we will initialize the Studio project as blank.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AddChapterResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/studio/projects/{jsonable_encoder(project_id)}/chapters",
            method="POST",
            json={
                "name": name,
                "from_url": from_url,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddChapterResponseModel,
                    construct_type(
                        type_=AddChapterResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, project_id: str, chapter_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ChapterWithContentResponseModel]:
        """
        Returns information about a specific chapter.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        chapter_id : str
            The ID of the chapter to be used. You can use the [List project chapters](/docs/api-reference/studio/get-chapters) endpoint to list all the available chapters.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ChapterWithContentResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/studio/projects/{jsonable_encoder(project_id)}/chapters/{jsonable_encoder(chapter_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ChapterWithContentResponseModel,
                    construct_type(
                        type_=ChapterWithContentResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        project_id: str,
        chapter_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        content: typing.Optional[ChapterContentInputModel] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[EditChapterResponseModel]:
        """
        Updates a chapter.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        chapter_id : str
            The ID of the chapter to be used. You can use the [List project chapters](/docs/api-reference/studio/get-chapters) endpoint to list all the available chapters.

        name : typing.Optional[str]
            The name of the chapter, used for identification only.

        content : typing.Optional[ChapterContentInputModel]
            The chapter content to use.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EditChapterResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/studio/projects/{jsonable_encoder(project_id)}/chapters/{jsonable_encoder(chapter_id)}",
            method="POST",
            json={
                "name": name,
                "content": convert_and_respect_annotation_metadata(
                    object_=content, annotation=ChapterContentInputModel, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EditChapterResponseModel,
                    construct_type(
                        type_=EditChapterResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self, project_id: str, chapter_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeleteChapterResponseModel]:
        """
        Deletes a chapter.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        chapter_id : str
            The ID of the chapter to be used. You can use the [List project chapters](/docs/api-reference/studio/get-chapters) endpoint to list all the available chapters.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteChapterResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/studio/projects/{jsonable_encoder(project_id)}/chapters/{jsonable_encoder(chapter_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteChapterResponseModel,
                    construct_type(
                        type_=DeleteChapterResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def convert(
        self, project_id: str, chapter_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ConvertChapterResponseModel]:
        """
        Starts conversion of a specific chapter.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        chapter_id : str
            The ID of the chapter to be used. You can use the [List project chapters](/docs/api-reference/studio/get-chapters) endpoint to list all the available chapters.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ConvertChapterResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/studio/projects/{jsonable_encoder(project_id)}/chapters/{jsonable_encoder(chapter_id)}/convert",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConvertChapterResponseModel,
                    construct_type(
                        type_=ConvertChapterResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawChaptersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, project_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetChaptersResponse]:
        """
        Returns a list of a Studio project's chapters.

        Parameters
        ----------
        project_id : str
            The ID of the Studio project.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetChaptersResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/studio/projects/{jsonable_encoder(project_id)}/chapters",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetChaptersResponse,
                    construct_type(
                        type_=GetChaptersResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        project_id: str,
        *,
        name: str,
        from_url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AddChapterResponseModel]:
        """
        Creates a new chapter either as blank or from a URL.

        Parameters
        ----------
        project_id : str
            The ID of the Studio project.

        name : str
            The name of the chapter, used for identification only.

        from_url : typing.Optional[str]
            An optional URL from which we will extract content to initialize the Studio project. If this is set, 'from_url' and 'from_content' must be null. If neither 'from_url', 'from_document', 'from_content' are provided we will initialize the Studio project as blank.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AddChapterResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/studio/projects/{jsonable_encoder(project_id)}/chapters",
            method="POST",
            json={
                "name": name,
                "from_url": from_url,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddChapterResponseModel,
                    construct_type(
                        type_=AddChapterResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, project_id: str, chapter_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ChapterWithContentResponseModel]:
        """
        Returns information about a specific chapter.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        chapter_id : str
            The ID of the chapter to be used. You can use the [List project chapters](/docs/api-reference/studio/get-chapters) endpoint to list all the available chapters.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ChapterWithContentResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/studio/projects/{jsonable_encoder(project_id)}/chapters/{jsonable_encoder(chapter_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ChapterWithContentResponseModel,
                    construct_type(
                        type_=ChapterWithContentResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        project_id: str,
        chapter_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        content: typing.Optional[ChapterContentInputModel] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[EditChapterResponseModel]:
        """
        Updates a chapter.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        chapter_id : str
            The ID of the chapter to be used. You can use the [List project chapters](/docs/api-reference/studio/get-chapters) endpoint to list all the available chapters.

        name : typing.Optional[str]
            The name of the chapter, used for identification only.

        content : typing.Optional[ChapterContentInputModel]
            The chapter content to use.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EditChapterResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/studio/projects/{jsonable_encoder(project_id)}/chapters/{jsonable_encoder(chapter_id)}",
            method="POST",
            json={
                "name": name,
                "content": convert_and_respect_annotation_metadata(
                    object_=content, annotation=ChapterContentInputModel, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EditChapterResponseModel,
                    construct_type(
                        type_=EditChapterResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, project_id: str, chapter_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeleteChapterResponseModel]:
        """
        Deletes a chapter.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        chapter_id : str
            The ID of the chapter to be used. You can use the [List project chapters](/docs/api-reference/studio/get-chapters) endpoint to list all the available chapters.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteChapterResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/studio/projects/{jsonable_encoder(project_id)}/chapters/{jsonable_encoder(chapter_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteChapterResponseModel,
                    construct_type(
                        type_=DeleteChapterResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def convert(
        self, project_id: str, chapter_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ConvertChapterResponseModel]:
        """
        Starts conversion of a specific chapter.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        chapter_id : str
            The ID of the chapter to be used. You can use the [List project chapters](/docs/api-reference/studio/get-chapters) endpoint to list all the available chapters.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ConvertChapterResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/studio/projects/{jsonable_encoder(project_id)}/chapters/{jsonable_encoder(chapter_id)}/convert",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConvertChapterResponseModel,
                    construct_type(
                        type_=ConvertChapterResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.datetime_utils import serialize_datetime
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.create_order_request import CreateOrderRequest
from ...types.create_order_response import CreateOrderResponse
from ...types.list_orders_response import ListOrdersResponse
from ...types.order_id import OrderId
from ...types.order_request_state import OrderRequestState
from ...types.order_response import OrderResponse
from ...types.submit_order_response import SubmitOrderResponse
from ...types.update_order_request import UpdateOrderRequest
from ...types.update_order_response import UpdateOrderResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawOrdersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        page_size: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        status: typing.Optional[typing.Union[OrderRequestState, typing.Sequence[OrderRequestState]]] = None,
        start_date: typing.Optional[dt.datetime] = None,
        end_date: typing.Optional[dt.datetime] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ListOrdersResponse]:
        """
        Lists Productions orders in the workspace. Supports filtering by status and date range, with pagination.

        Parameters
        ----------
        page_size : typing.Optional[int]
            Maximum number of orders to return per page.

        offset : typing.Optional[int]
            Number of orders to skip for pagination.

        status : typing.Optional[typing.Union[OrderRequestState, typing.Sequence[OrderRequestState]]]
            Filter orders by one or more statuses.

        start_date : typing.Optional[dt.datetime]
            Filter orders created on or after this date.

        end_date : typing.Optional[dt.datetime]
            Filter orders created on or before this date.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListOrdersResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/productions/orders",
            method="GET",
            params={
                "page_size": page_size,
                "offset": offset,
                "status": status,
                "start_date": serialize_datetime(start_date) if start_date is not None else None,
                "end_date": serialize_datetime(end_date) if end_date is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListOrdersResponse,
                    construct_type(
                        type_=ListOrdersResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        request: typing.Optional[CreateOrderRequest] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateOrderResponse]:
        """
        Creates a new Productions order in the workspace. The order starts in the open state and can be configured with items before submission.

        Parameters
        ----------
        request : typing.Optional[CreateOrderRequest]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateOrderResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/productions/orders",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=CreateOrderRequest, direction="write"
            ),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateOrderResponse,
                    construct_type(
                        type_=CreateOrderResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, order_id: OrderId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[OrderResponse]:
        """
        Retrieves full details for a Productions order.

        Quote and pricing information may not be available immediately; if you wish to see the quote before submission, you may need to poll the order details until it is ready.

        Parameters
        ----------
        order_id : OrderId
            The ID of the order.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[OrderResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/productions/orders/{jsonable_encoder(order_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    OrderResponse,
                    construct_type(
                        type_=OrderResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self, order_id: OrderId, *, request: UpdateOrderRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[UpdateOrderResponse]:
        """
        Updates an open order.

        Parameters
        ----------
        order_id : OrderId
            The ID of the order.

        request : UpdateOrderRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateOrderResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/productions/orders/{jsonable_encoder(order_id)}",
            method="PATCH",
            json={
                "request": convert_and_respect_annotation_metadata(
                    object_=request, annotation=UpdateOrderRequest, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateOrderResponse,
                    construct_type(
                        type_=UpdateOrderResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def submit(
        self, order_id: OrderId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[SubmitOrderResponse]:
        """
        Submits an open order for processing. The order must have at least one item. Once submitted, items can no longer be modified.

        Upon submission, the workspace will be charged for the order. The quote is based on information extracted from the uploaded media, such as its duration. The quote may not be available immediately; if you wish to see the quote before submission, you may need to poll the order details until the quote is ready.

        Parameters
        ----------
        order_id : OrderId
            The ID of the order.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SubmitOrderResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/productions/orders/{jsonable_encoder(order_id)}/submit",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SubmitOrderResponse,
                    construct_type(
                        type_=SubmitOrderResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawOrdersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        page_size: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        status: typing.Optional[typing.Union[OrderRequestState, typing.Sequence[OrderRequestState]]] = None,
        start_date: typing.Optional[dt.datetime] = None,
        end_date: typing.Optional[dt.datetime] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ListOrdersResponse]:
        """
        Lists Productions orders in the workspace. Supports filtering by status and date range, with pagination.

        Parameters
        ----------
        page_size : typing.Optional[int]
            Maximum number of orders to return per page.

        offset : typing.Optional[int]
            Number of orders to skip for pagination.

        status : typing.Optional[typing.Union[OrderRequestState, typing.Sequence[OrderRequestState]]]
            Filter orders by one or more statuses.

        start_date : typing.Optional[dt.datetime]
            Filter orders created on or after this date.

        end_date : typing.Optional[dt.datetime]
            Filter orders created on or before this date.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListOrdersResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/productions/orders",
            method="GET",
            params={
                "page_size": page_size,
                "offset": offset,
                "status": status,
                "start_date": serialize_datetime(start_date) if start_date is not None else None,
                "end_date": serialize_datetime(end_date) if end_date is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListOrdersResponse,
                    construct_type(
                        type_=ListOrdersResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        request: typing.Optional[CreateOrderRequest] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateOrderResponse]:
        """
        Creates a new Productions order in the workspace. The order starts in the open state and can be configured with items before submission.

        Parameters
        ----------
        request : typing.Optional[CreateOrderRequest]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateOrderResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/productions/orders",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=CreateOrderRequest, direction="write"
            ),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateOrderResponse,
                    construct_type(
                        type_=CreateOrderResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, order_id: OrderId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[OrderResponse]:
        """
        Retrieves full details for a Productions order.

        Quote and pricing information may not be available immediately; if you wish to see the quote before submission, you may need to poll the order details until it is ready.

        Parameters
        ----------
        order_id : OrderId
            The ID of the order.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[OrderResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/productions/orders/{jsonable_encoder(order_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    OrderResponse,
                    construct_type(
                        type_=OrderResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self, order_id: OrderId, *, request: UpdateOrderRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[UpdateOrderResponse]:
        """
        Updates an open order.

        Parameters
        ----------
        order_id : OrderId
            The ID of the order.

        request : UpdateOrderRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateOrderResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/productions/orders/{jsonable_encoder(order_id)}",
            method="PATCH",
            json={
                "request": convert_and_respect_annotation_metadata(
                    object_=request, annotation=UpdateOrderRequest, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateOrderResponse,
                    construct_type(
                        type_=UpdateOrderResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def submit(
        self, order_id: OrderId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[SubmitOrderResponse]:
        """
        Submits an open order for processing. The order must have at least one item. Once submitted, items can no longer be modified.

        Upon submission, the workspace will be charged for the order. The quote is based on information extracted from the uploaded media, such as its duration. The quote may not be available immediately; if you wish to see the quote before submission, you may need to poll the order details until the quote is ready.

        Parameters
        ----------
        order_id : OrderId
            The ID of the order.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SubmitOrderResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/productions/orders/{jsonable_encoder(order_id)}/submit",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SubmitOrderResponse,
                    construct_type(
                        type_=SubmitOrderResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, update_forward_refs
from ..core.unchecked_base_model import UncheckedBaseModel
from .conversation_history_transcript_common_model_input import ConversationHistoryTranscriptCommonModelInput
from .conversation_initiation_source import ConversationInitiationSource
from .dynamic_variable_value_type_input import DynamicVariableValueTypeInput
from .llm import Llm
from .simulation_tool_mock_behavior_config import SimulationToolMockBehaviorConfig
from .test_from_conversation_metadata_input import TestFromConversationMetadataInput


class UpdateSimulationTestRequest(UncheckedBaseModel):
    from_conversation_metadata: typing.Optional[TestFromConversationMetadataInput] = pydantic.Field(default=None)
    """
    Metadata of a conversation this test was created from (if applicable).
    """

    dynamic_variables: typing.Optional[typing.Dict[str, typing.Optional[DynamicVariableValueTypeInput]]] = (
        pydantic.Field(default=None)
    )
    """
    Dynamic variables to replace in the agent config during testing
    """

    chat_history: typing.Optional[typing.List[ConversationHistoryTranscriptCommonModelInput]] = None
    conversation_initiation_source: typing.Optional[ConversationInitiationSource] = pydantic.Field(default=None)
    """
    Simulate the test as if the conversation originated from this channel.
    """

    success_condition: typing.Optional[str] = pydantic.Field(default=None)
    """
    Deprecated legacy single success criterion. Use success_conditions instead. At least one of success_condition or success_conditions is required.
    """

    success_conditions: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of prompts that evaluate whether the simulation was successful. If provided, all criteria are evaluated and merged into a final result. Capped at the maximum number of evaluation criteria.
    """

    simulation_scenario: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description of the simulation scenario and user persona for simulation tests.
    """

    simulation_max_turns: typing.Optional[int] = pydantic.Field(default=None)
    """
    Maximum number of conversation turns for simulation tests.
    """

    simulation_environment: typing.Optional[str] = pydantic.Field(default=None)
    """
    The environment to use when running this simulation test. If not provided, defaults to 'production'.
    """

    tool_mock_config: typing.Optional[SimulationToolMockBehaviorConfig] = pydantic.Field(default=None)
    """
    Configuration for which tools to mock and fallback behavior.
    """

    evaluation_model: typing.Optional[Llm] = pydantic.Field(default=None)
    """
    LLM model to use for evaluating simulation results. Defaults to Claude Sonnet 4.6.
    """

    simulated_user_model: typing.Optional[Llm] = pydantic.Field(default=None)
    """
    LLM model for the simulated user. Defaults to Claude Sonnet 4.6.
    """

    name: str
    parent_folder_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the parent folder. If not provided, the test will be moved to the root level.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


update_forward_refs(UpdateSimulationTestRequest)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .feature_status_common_model import FeatureStatusCommonModel
from .tests_feature_usage_common_model import TestsFeatureUsageCommonModel
from .workflow_features_usage_common_model import WorkflowFeaturesUsageCommonModel


class FeaturesUsageCommonModel(UncheckedBaseModel):
    language_detection: typing.Optional[FeatureStatusCommonModel] = None
    transfer_to_agent: typing.Optional[FeatureStatusCommonModel] = None
    transfer_to_number: typing.Optional[FeatureStatusCommonModel] = None
    multivoice: typing.Optional[FeatureStatusCommonModel] = None
    dtmf_tones: typing.Optional[FeatureStatusCommonModel] = None
    external_mcp_servers: typing.Optional[FeatureStatusCommonModel] = None
    pii_zrm_workspace: typing.Optional[bool] = None
    pii_zrm_agent: typing.Optional[bool] = None
    tool_dynamic_variable_updates: typing.Optional[FeatureStatusCommonModel] = None
    is_livekit: typing.Optional[bool] = None
    voicemail_detection: typing.Optional[FeatureStatusCommonModel] = None
    dtmf_input: typing.Optional[FeatureStatusCommonModel] = None
    workflow: typing.Optional[WorkflowFeaturesUsageCommonModel] = None
    agent_testing: typing.Optional[TestsFeatureUsageCommonModel] = None
    versioning: typing.Optional[FeatureStatusCommonModel] = None
    file_input: typing.Optional[FeatureStatusCommonModel] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .batch_call_recipient_status import BatchCallRecipientStatus
from .conversation_initiation_client_data_internal import ConversationInitiationClientDataInternal


class OutboundCallRecipientResponseModel(UncheckedBaseModel):
    id: str
    phone_number: typing.Optional[str] = None
    whatsapp_user_id: typing.Optional[str] = None
    status: BatchCallRecipientStatus
    created_at_unix: int
    updated_at_unix: int
    conversation_id: typing.Optional[str] = None
    conversation_initiation_client_data: typing.Optional[ConversationInitiationClientDataInternal] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .document_refresh_response import (
        DocumentRefreshResponse,
        DocumentRefreshResponse_File,
        DocumentRefreshResponse_Folder,
        DocumentRefreshResponse_Text,
        DocumentRefreshResponse_Url,
    )
    from .document_update_file_response import (
        DocumentUpdateFileResponse,
        DocumentUpdateFileResponse_File,
        DocumentUpdateFileResponse_Folder,
        DocumentUpdateFileResponse_Text,
        DocumentUpdateFileResponse_Url,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "DocumentRefreshResponse": ".document_refresh_response",
    "DocumentRefreshResponse_File": ".document_refresh_response",
    "DocumentRefreshResponse_Folder": ".document_refresh_response",
    "DocumentRefreshResponse_Text": ".document_refresh_response",
    "DocumentRefreshResponse_Url": ".document_refresh_response",
    "DocumentUpdateFileResponse": ".document_update_file_response",
    "DocumentUpdateFileResponse_File": ".document_update_file_response",
    "DocumentUpdateFileResponse_Folder": ".document_update_file_response",
    "DocumentUpdateFileResponse_Text": ".document_update_file_response",
    "DocumentUpdateFileResponse_Url": ".document_update_file_response",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "DocumentRefreshResponse",
    "DocumentRefreshResponse_File",
    "DocumentRefreshResponse_Folder",
    "DocumentRefreshResponse_Text",
    "DocumentRefreshResponse_Url",
    "DocumentUpdateFileResponse",
    "DocumentUpdateFileResponse_File",
    "DocumentUpdateFileResponse_Folder",
    "DocumentUpdateFileResponse_Text",
    "DocumentUpdateFileResponse_Url",
]



# This file was auto-generated by Fern from our API Definition.

import typing

WebhookUsageType = typing.Union[
    typing.Literal[
        "ConvAI Agent Settings",
        "ConvAI Settings",
        "Voice Library Removal Notices",
        "Speech to Text",
        "Agent QA Evaluations",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .conversation_asr_usage_model import ConversationAsrUsageModel
from .conversation_tts_usage_model import ConversationTtsUsageModel
from .llm_category_usage import LlmCategoryUsage


class ConversationChargingCommonModel(UncheckedBaseModel):
    dev_discount: typing.Optional[bool] = None
    is_burst: typing.Optional[bool] = None
    tier: typing.Optional[str] = None
    llm_usage: typing.Optional[LlmCategoryUsage] = None
    llm_price: typing.Optional[float] = None
    llm_charge: typing.Optional[int] = None
    call_charge: typing.Optional[int] = None
    free_minutes_consumed: typing.Optional[float] = None
    free_llm_dollars_consumed: typing.Optional[float] = None
    tts_usage: typing.Optional[ConversationTtsUsageModel] = None
    asr_usage: typing.Optional[ConversationAsrUsageModel] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .resource_access_info import ResourceAccessInfo


class TestInvocationSummaryResponseModel(UncheckedBaseModel):
    id: str = pydantic.Field()
    """
    The ID of the test invocation
    """

    agent_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the agent this test invocation belongs to
    """

    branch_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the branch this test invocation was run on
    """

    created_at_unix_secs: int = pydantic.Field()
    """
    Creation time of the test invocation in unix seconds
    """

    test_run_count: int = pydantic.Field()
    """
    Number of test runs in this invocation
    """

    passed_count: int = pydantic.Field()
    """
    Number of test runs that passed
    """

    failed_count: int = pydantic.Field()
    """
    Number of test runs that failed
    """

    pending_count: int = pydantic.Field()
    """
    Number of test runs that are pending
    """

    title: str = pydantic.Field()
    """
    Title of the test invocation - either the single test name or count of tests
    """

    access_info: typing.Optional[ResourceAccessInfo] = pydantic.Field(default=None)
    """
    The access information of the test invocation
    """

    repeat_count: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of times each test was repeated in this invocation
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

BodyEditServiceAccountApiKeyV1ServiceAccountsServiceAccountUserIdApiKeysApiKeyIdPatchIsEnabled = typing.Union[
    bool, typing.Literal["no_update"]
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .clip_animation_enter_effect import ClipAnimationEnterEffect
from .clip_animation_exit_effect import ClipAnimationExitEffect


class ClipAnimation(UncheckedBaseModel):
    enter_effect: typing.Optional[ClipAnimationEnterEffect] = None
    enter_duration_ms: typing.Optional[int] = None
    exit_effect: typing.Optional[ClipAnimationExitEffect] = None
    exit_duration_ms: typing.Optional[int] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



import json

import pytest

from elevenlabs.music_custom import MusicClient, _find_audio_start

SAMPLE_JSON = {"compositionPlan": {"sections": []}, "songMetadata": {"title": "Test"}}
SAMPLE_AUDIO = bytes(range(256)) * 4


def _build_multipart(line_ending: bytes) -> bytes:
    """Build a realistic multipart response with the given line ending."""
    boundary = b"--boundary123"
    nl = line_ending

    parts = []
    parts.append(boundary + nl)
    parts.append(b"Content-Type: application/json" + nl)
    parts.append(nl)
    parts.append(json.dumps(SAMPLE_JSON).encode("utf-8") + nl)
    parts.append(boundary + nl)
    parts.append(b"Content-Type: audio/mpeg" + nl)
    parts.append(b'Content-Disposition: attachment; filename="test_song.mp3"' + nl)
    parts.append(nl)
    parts.append(SAMPLE_AUDIO)
    parts.append(nl + boundary + b"--" + nl)

    return b"".join(parts)


class TestFindAudioStart:
    def test_crlf(self):
        data = b"Header: value\r\n\r\naudio data here"
        assert _find_audio_start(data, 0) == data.index(b"audio")

    def test_lf(self):
        data = b"Header: value\n\naudio data here"
        assert _find_audio_start(data, 0) == data.index(b"audio")

    def test_crlf_preferred_over_lf(self):
        data = b"Header: value\r\n\r\naudio data here"
        pos = _find_audio_start(data, 0)
        assert data[pos : pos + 5] == b"audio"

    def test_raises_when_no_separator(self):
        with pytest.raises(ValueError, match="header/body separator"):
            _find_audio_start(b"no blank line here", 0)

    def test_start_offset_respected(self):
        data = b"skip\n\nHeader: value\r\n\r\naudio"
        pos = _find_audio_start(data, 6)
        assert data[pos : pos + 5] == b"audio"


class TestParseMultipart:
    @staticmethod
    def _parse(data: bytes):
        return MusicClient._parse_multipart(None, iter([data]))  # type: ignore[arg-type]

    def test_crlf_line_endings(self):
        data = _build_multipart(b"\r\n")
        result = self._parse(data)
        assert result.audio.startswith(SAMPLE_AUDIO)
        assert result.json == SAMPLE_JSON
        assert result.filename == "test_song.mp3"

    def test_lf_line_endings(self):
        data = _build_multipart(b"\n")
        result = self._parse(data)
        assert result.audio.startswith(SAMPLE_AUDIO)
        assert result.json == SAMPLE_JSON
        assert result.filename == "test_song.mp3"

    def test_audio_bytes_not_corrupted(self):
        for nl in [b"\r\n", b"\n"]:
            data = _build_multipart(nl)
            result = self._parse(data)
            assert result.audio[:4] == SAMPLE_AUDIO[:4], (
                f"First 4 bytes mismatch with {nl!r} line endings — possible byte offset error"
            )
            assert result.audio[: len(SAMPLE_AUDIO)] == SAMPLE_AUDIO



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .... import core
from ....core.api_error import ApiError
from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.http_response import AsyncHttpResponse, HttpResponse
from ....core.jsonable_encoder import jsonable_encoder
from ....core.request_options import RequestOptions
from ....core.unchecked_base_model import construct_type
from ....errors.unprocessable_entity_error import UnprocessableEntityError
from ....types.request_pvc_manual_verification_response_model import RequestPvcManualVerificationResponseModel

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawVerificationClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def request(
        self,
        voice_id: str,
        *,
        files: typing.List[core.File],
        extra_text: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[RequestPvcManualVerificationResponseModel]:
        """
        Request manual verification for a PVC voice.

        Parameters
        ----------
        voice_id : str
            Voice ID to be used, you can use https://api.elevenlabs.io/v1/voices to list all the available voices.

        files : typing.List[core.File]
            See core.File for more documentation

        extra_text : typing.Optional[str]
            Extra text to be used in the manual verification process.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RequestPvcManualVerificationResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/voices/pvc/{jsonable_encoder(voice_id)}/verification",
            method="POST",
            data={
                "extra_text": extra_text,
            },
            files={
                "files": files,
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RequestPvcManualVerificationResponseModel,
                    construct_type(
                        type_=RequestPvcManualVerificationResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawVerificationClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def request(
        self,
        voice_id: str,
        *,
        files: typing.List[core.File],
        extra_text: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[RequestPvcManualVerificationResponseModel]:
        """
        Request manual verification for a PVC voice.

        Parameters
        ----------
        voice_id : str
            Voice ID to be used, you can use https://api.elevenlabs.io/v1/voices to list all the available voices.

        files : typing.List[core.File]
            See core.File for more documentation

        extra_text : typing.Optional[str]
            Extra text to be used in the manual verification process.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RequestPvcManualVerificationResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/voices/pvc/{jsonable_encoder(voice_id)}/verification",
            method="POST",
            data={
                "extra_text": extra_text,
            },
            files={
                "files": files,
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RequestPvcManualVerificationResponseModel,
                    construct_type(
                        type_=RequestPvcManualVerificationResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

ColumnUnit = typing.Union[
    typing.Literal["ms", "s", "min", "duration", "credits", "usd", "eur", "inr", "pln", "ratio", "rating"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel


class GenerationConfig(UncheckedBaseModel):
    chunk_length_schedule: typing.Optional[typing.List[float]] = pydantic.Field(default=None)
    """
    This is an advanced setting that most users shouldn't need to use. It relates to our
    generation schedule.
    
    Our WebSocket service incorporates a buffer system designed to optimize the Time To First Byte (TTFB) while maintaining high-quality streaming.
    
    All text sent to the WebSocket endpoint is added to this buffer and only when that buffer reaches a certain size is an audio generation attempted. This is because our model provides higher quality audio when the model has longer inputs, and can deduce more context about how the text should be delivered.
    
    The buffer ensures smooth audio data delivery and is automatically emptied with a final audio generation either when the stream is closed, or upon sending a `flush` command. We have advanced settings for changing the chunk schedule, which can improve latency at the cost of quality by generating audio more frequently with smaller text inputs.
    
    The `chunk_length_schedule` determines the minimum amount of text that needs to be sent and present in our
    buffer before audio starts being generated. This is to maximise the amount of context available to
    the model to improve audio quality, whilst balancing latency of the returned audio chunks.
    
    The default value for `chunk_length_schedule` is: [120, 160, 250, 290].
    
    This means that the first chunk of audio will not be generated until you send text that
    totals at least 120 characters long. The next chunk of audio will only be generated once a
    further 160 characters have been sent. The third audio chunk will be generated after the
    next 250 characters. Then the fourth, and beyond, will be generated in sets of at least 290 characters.
    
    Customize this array to suit your needs. If you want to generate audio more frequently
    to optimise latency, you can reduce the values in the array. Note that setting the values
    too low may result in lower quality audio. Please test and adjust as needed.
    
    Each item should be in the range 50-500.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...types.close_payload import ClosePayload
from ...types.error_payload import ErrorPayload
from ...types.init_payload import InitPayload
from ...types.ping_payload import PingPayload
from ...types.user_transcript_payload import UserTranscriptPayload

ReceiveUpstreamMessage = typing.Union[InitPayload, UserTranscriptPayload, PingPayload, ClosePayload, ErrorPayload]



"""SpeechEngineServer — standalone WebSocket server for Speech Engine."""

import asyncio
import http
import os
import typing

from .session import SpeechEngineSession, _make_log, _wire_handlers
from .types import WebSocketLike


class SpeechEngineServer:
    """Standalone WebSocket server that produces :class:`SpeechEngineSession`
    instances for each incoming connection from the ElevenLabs Speech Engine
    API.

    Every incoming connection is verified against the ElevenLabs API using
    the configured API key before being accepted.

    Example::

        server = SpeechEngineServer(
            port=3001,
            api_key="sk_...",
            debug=True,
            on_transcript=handle_transcript,
        )
        await server.serve()
    """

    def __init__(
        self,
        *,
        port: int = 3001,
        path: typing.Optional[str] = None,
        api_key: typing.Optional[str] = None,
        debug: bool = False,
        **handlers: typing.Any,
    ) -> None:
        self._port = port
        self._path = path
        self._api_key = api_key
        self._debug = debug
        self._handlers = handlers
        self._stop_event = None  # type: typing.Optional[asyncio.Event]
        self._server = None  # type: typing.Any
        self._log = _make_log(debug)

    def handle_connection(self, ws: WebSocketLike) -> SpeechEngineSession:
        """Wrap *ws* in a :class:`SpeechEngineSession` with the server's
        handlers wired up.

        Use this when you manage your own WebSocket server and want to wrap
        individual connections.  The returned session's :meth:`run` must
        still be awaited by the caller.
        """
        self._log("creating new session")
        session = SpeechEngineSession(ws, debug=self._debug)
        _wire_handlers(session, self._handlers)
        return session

    async def serve(self) -> None:
        """Start the WebSocket server.  Blocks until :meth:`stop` is called."""
        from .resource import verify_speech_engine_jwt  # noqa: E402

        import websockets  # noqa: E402 — keep import lazy

        api_key = self._api_key or os.environ.get("ELEVENLABS_API_KEY")
        if not api_key:
            raise RuntimeError(
                "SpeechEngineServer requires an API key to verify incoming "
                "connections. Pass api_key= or set the ELEVENLABS_API_KEY "
                "environment variable."
            )

        self._stop_event = asyncio.Event()

        def _process_request(
            connection: typing.Any, request: typing.Any
        ) -> typing.Any:
            if self._path is not None and request.path != self._path:
                self._log(
                    "rejected connection — path mismatch: "
                    "expected %s, got %s",
                    self._path,
                    request.path,
                )
                return connection.respond(
                    http.HTTPStatus.NOT_FOUND, "not found\n"
                )

            header_value = request.headers.get(
                "x-elevenlabs-speech-engine-authorization"
            )
            if not header_value:
                self._log(
                    "rejected connection — missing "
                    "X-Elevenlabs-Speech-Engine-Authorization header"
                )
                return connection.respond(
                    http.HTTPStatus.UNAUTHORIZED,
                    "missing authorization header\n",
                )

            try:
                verify_speech_engine_jwt(header_value, api_key)
            except ValueError as e:
                self._log("rejected connection — %s", e)
                return connection.respond(
                    http.HTTPStatus.UNAUTHORIZED,
                    "authorization failed\n",
                )

            return None

        async def _handler(websocket: typing.Any) -> None:
            self._log("verified connection, accepting WebSocket")
            session = self.handle_connection(websocket)
            await session.run()

        self._server = await websockets.serve(  # type: ignore[attr-defined]
            _handler,
            "",
            self._port,
            process_request=_process_request,
        )
        self._log("speech engine server listening on port %d", self._port)
        try:
            await self._stop_event.wait()
        finally:
            self._server.close()
            await self._server.wait_closed()

    async def stop(self) -> None:
        """Signal the server to shut down gracefully."""
        if self._stop_event is not None:
            self._stop_event.set()



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, update_forward_refs
from ..core.unchecked_base_model import UncheckedBaseModel
from .literal_json_schema_property import LiteralJsonSchemaProperty
from .query_params_json_schema import QueryParamsJsonSchema
from .webhook_tool_api_schema_config_input_auth_connection import WebhookToolApiSchemaConfigInputAuthConnection
from .webhook_tool_api_schema_config_input_content_type import WebhookToolApiSchemaConfigInputContentType
from .webhook_tool_api_schema_config_input_method import WebhookToolApiSchemaConfigInputMethod
from .webhook_tool_api_schema_config_input_request_headers_value import (
    WebhookToolApiSchemaConfigInputRequestHeadersValue,
)


class WebhookToolApiSchemaConfigInput(UncheckedBaseModel):
    request_headers: typing.Optional[typing.Dict[str, WebhookToolApiSchemaConfigInputRequestHeadersValue]] = (
        pydantic.Field(default=None)
    )
    """
    Headers that should be included in the request
    """

    url: str = pydantic.Field()
    """
    The URL that the webhook will be sent to. May include path parameters, e.g. https://example.com/agents/{agent_id}
    """

    method: typing.Optional[WebhookToolApiSchemaConfigInputMethod] = pydantic.Field(default=None)
    """
    The HTTP method to use for the webhook
    """

    path_params_schema: typing.Optional[typing.Dict[str, LiteralJsonSchemaProperty]] = pydantic.Field(default=None)
    """
    Schema for path parameters, if any. The keys should match the placeholders in the URL.
    """

    query_params_schema: typing.Optional[QueryParamsJsonSchema] = pydantic.Field(default=None)
    """
    Schema for any query params, if any. These will be added to end of the URL as query params. Note: properties in a query param must all be literal types
    """

    request_body_schema: typing.Optional["ObjectJsonSchemaPropertyInput"] = pydantic.Field(default=None)
    """
    Schema for the body parameters, if any. Used for POST/PATCH/PUT requests. The schema should be an object which will be sent as the json body
    """

    response_body_schema: typing.Optional["ObjectJsonSchemaPropertyInput"] = pydantic.Field(default=None)
    """
    Schema describing the expected response body structure. For documentation only; not surfaced to the LLM.
    """

    content_type: typing.Optional[WebhookToolApiSchemaConfigInputContentType] = pydantic.Field(default=None)
    """
    Content type for the request body. Only applies to POST/PUT/PATCH requests.
    """

    auth_resolved_params: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    URL placeholders resolved from the auth connection (e.g. secrets injected via UrlSecretAuthConnection) rather than from path_params_schema.
    """

    auth_connection: typing.Optional[WebhookToolApiSchemaConfigInputAuthConnection] = pydantic.Field(default=None)
    """
    Optional auth connection to use for authentication with this webhook
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .object_json_schema_property_input import ObjectJsonSchemaPropertyInput  # noqa: E402, I001

update_forward_refs(WebhookToolApiSchemaConfigInput)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .branch_protection_status import BranchProtectionStatus
from .resource_access_info import ResourceAccessInfo


class AgentBranchSummary(UncheckedBaseModel):
    id: str
    name: str
    agent_id: str
    description: str
    created_at: int
    last_committed_at: int
    is_archived: bool
    protection_status: typing.Optional[BranchProtectionStatus] = None
    access_info: typing.Optional[ResourceAccessInfo] = pydantic.Field(default=None)
    """
    Access information for the branch
    """

    current_live_percentage: typing.Optional[float] = pydantic.Field(default=None)
    """
    Percentage of traffic live on the branch
    """

    parent_branch_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the parent branch
    """

    draft_exists: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether a draft exists for the branch
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.request_options import RequestOptions
from ....types.agent_deployment_request import AgentDeploymentRequest
from ....types.agent_deployment_response import AgentDeploymentResponse
from .raw_client import AsyncRawDeploymentsClient, RawDeploymentsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DeploymentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDeploymentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawDeploymentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDeploymentsClient
        """
        return self._raw_client

    def create(
        self,
        agent_id: str,
        *,
        deployment_request: AgentDeploymentRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AgentDeploymentResponse:
        """
        Create a new deployment for an agent

        Parameters
        ----------
        agent_id : str
            The id of an agent. This is returned on agent creation.

        deployment_request : AgentDeploymentRequest
            Request to create a new deployment

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentDeploymentResponse
            Successful Response

        Examples
        --------
        from elevenlabs import (
            AgentDeploymentPercentageStrategy,
            AgentDeploymentRequest,
            AgentDeploymentRequestItem,
            ElevenLabs,
        )

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.agents.deployments.create(
            agent_id="agent_3701k3ttaq12ewp8b7qv5rfyszkz",
            deployment_request=AgentDeploymentRequest(
                requests=[
                    AgentDeploymentRequestItem(
                        branch_id="agtbrch_8901k4t9z5defmb8vh3e9361y7nj",
                        deployment_strategy=AgentDeploymentPercentageStrategy(
                            traffic_percentage=0.5,
                        ),
                    )
                ],
            ),
        )
        """
        _response = self._raw_client.create(
            agent_id, deployment_request=deployment_request, request_options=request_options
        )
        return _response.data


class AsyncDeploymentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDeploymentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawDeploymentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDeploymentsClient
        """
        return self._raw_client

    async def create(
        self,
        agent_id: str,
        *,
        deployment_request: AgentDeploymentRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AgentDeploymentResponse:
        """
        Create a new deployment for an agent

        Parameters
        ----------
        agent_id : str
            The id of an agent. This is returned on agent creation.

        deployment_request : AgentDeploymentRequest
            Request to create a new deployment

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentDeploymentResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import (
            AgentDeploymentPercentageStrategy,
            AgentDeploymentRequest,
            AgentDeploymentRequestItem,
            AsyncElevenLabs,
        )

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.agents.deployments.create(
                agent_id="agent_3701k3ttaq12ewp8b7qv5rfyszkz",
                deployment_request=AgentDeploymentRequest(
                    requests=[
                        AgentDeploymentRequestItem(
                            branch_id="agtbrch_8901k4t9z5defmb8vh3e9361y7nj",
                            deployment_strategy=AgentDeploymentPercentageStrategy(
                                traffic_percentage=0.5,
                            ),
                        )
                    ],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            agent_id, deployment_request=deployment_request, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .search_highlight_segment import SearchHighlightSegment


class MessagesSearchResult(UncheckedBaseModel):
    """
    transcript_index: index of the message in the conversation transcript
    chunk_text: text of the transcript; transcript messages if very long could have several chunks.
    chunk_highlights: chunk_text split into matched/unmatched segments for highlighting.
        Only populated for keyword/text search, not semantic search.
    score: similarity score of the message to the search query
    """

    conversation_id: str
    agent_id: str
    agent_name: typing.Optional[str] = None
    transcript_index: int
    chunk_text: str
    chunk_highlights: typing.Optional[typing.List[SearchHighlightSegment]] = None
    score: float
    conversation_start_time_unix_secs: int

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

WorkspaceAnalyticsQueryResponseModelColumnTypesItem = typing.Union[
    typing.Literal["String", "Float", "DateTime", "Int", "Bool", "JSON", "Map"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .agent_definition_source import AgentDefinitionSource
from .async_conversation_metadata import AsyncConversationMetadata
from .authorization_method import AuthorizationMethod
from .conversation_charging_common_model import ConversationChargingCommonModel
from .conversation_deletion_settings import ConversationDeletionSettings
from .conversation_history_batch_call_model import ConversationHistoryBatchCallModel
from .conversation_history_eleven_assistant_common_model import ConversationHistoryElevenAssistantCommonModel
from .conversation_history_error_common_model import ConversationHistoryErrorCommonModel
from .conversation_history_feedback_common_model import ConversationHistoryFeedbackCommonModel
from .conversation_history_metadata_common_model_phone_call import ConversationHistoryMetadataCommonModelPhoneCall
from .conversation_history_rag_usage_common_model import ConversationHistoryRagUsageCommonModel
from .conversation_initiation_source import ConversationInitiationSource
from .conversation_voice_reward_model import ConversationVoiceRewardModel
from .features_usage_common_model import FeaturesUsageCommonModel
from .sms_conversation_info import SmsConversationInfo
from .whats_app_conversation_info import WhatsAppConversationInfo


class ConversationHistoryMetadataCommonModel(UncheckedBaseModel):
    start_time_unix_secs: int
    accepted_time_unix_secs: typing.Optional[int] = None
    call_duration_secs: int
    cost: typing.Optional[int] = None
    deletion_settings: typing.Optional[ConversationDeletionSettings] = None
    feedback: typing.Optional[ConversationHistoryFeedbackCommonModel] = None
    authorization_method: typing.Optional[AuthorizationMethod] = None
    charging: typing.Optional[ConversationChargingCommonModel] = None
    phone_call: typing.Optional[ConversationHistoryMetadataCommonModelPhoneCall] = None
    batch_call: typing.Optional[ConversationHistoryBatchCallModel] = None
    termination_reason: typing.Optional[str] = None
    error: typing.Optional[ConversationHistoryErrorCommonModel] = None
    warnings: typing.Optional[typing.List[str]] = None
    main_language: typing.Optional[str] = None
    rag_usage: typing.Optional[ConversationHistoryRagUsageCommonModel] = None
    text_only: typing.Optional[bool] = None
    features_usage: typing.Optional[FeaturesUsageCommonModel] = None
    eleven_assistant: typing.Optional[ConversationHistoryElevenAssistantCommonModel] = None
    initiator_id: typing.Optional[str] = None
    conversation_initiation_source: typing.Optional[ConversationInitiationSource] = None
    conversation_initiation_source_version: typing.Optional[str] = None
    timezone: typing.Optional[str] = None
    async_metadata: typing.Optional[AsyncConversationMetadata] = None
    whatsapp: typing.Optional[WhatsAppConversationInfo] = None
    sms: typing.Optional[SmsConversationInfo] = None
    agent_created_from: typing.Optional[AgentDefinitionSource] = None
    agent_last_updated_from: typing.Optional[AgentDefinitionSource] = None
    voice_rewards: typing.Optional[typing.List[ConversationVoiceRewardModel]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.dubbing_transcripts_response_model import DubbingTranscriptsResponseModel
from .raw_client import AsyncRawTranscriptsClient, RawTranscriptsClient
from .types.transcripts_get_request_format_type import TranscriptsGetRequestFormatType


class TranscriptsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTranscriptsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTranscriptsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTranscriptsClient
        """
        return self._raw_client

    def get(
        self,
        dubbing_id: str,
        language_code: str,
        format_type: TranscriptsGetRequestFormatType,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DubbingTranscriptsResponseModel:
        """
        Fetch the transcript for one of the languages in a dub.

        Parameters
        ----------
        dubbing_id : str
            ID of the dubbing project.

        language_code : str
            ISO-693 language code to retrieve the transcript for. Use 'source' to fetch the transcript of the original media.

        format_type : TranscriptsGetRequestFormatType
            Format to return transcript in. For subtitles use either 'srt' or 'webvtt', and for a full transcript use 'json'. The 'json' format is not yet supported for Dubbing Studio.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DubbingTranscriptsResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.dubbing.transcripts.get(
            dubbing_id="dubbing_id",
            language_code="source",
            format_type="srt",
        )
        """
        _response = self._raw_client.get(dubbing_id, language_code, format_type, request_options=request_options)
        return _response.data


class AsyncTranscriptsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTranscriptsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTranscriptsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTranscriptsClient
        """
        return self._raw_client

    async def get(
        self,
        dubbing_id: str,
        language_code: str,
        format_type: TranscriptsGetRequestFormatType,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DubbingTranscriptsResponseModel:
        """
        Fetch the transcript for one of the languages in a dub.

        Parameters
        ----------
        dubbing_id : str
            ID of the dubbing project.

        language_code : str
            ISO-693 language code to retrieve the transcript for. Use 'source' to fetch the transcript of the original media.

        format_type : TranscriptsGetRequestFormatType
            Format to return transcript in. For subtitles use either 'srt' or 'webvtt', and for a full transcript use 'json'. The 'json' format is not yet supported for Dubbing Studio.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DubbingTranscriptsResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.dubbing.transcripts.get(
                dubbing_id="dubbing_id",
                language_code="source",
                format_type="srt",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(dubbing_id, language_code, format_type, request_options=request_options)
        return _response.data



"""
Tests for ClientTools custom event loop functionality.
"""

import asyncio
import pytest
import threading
import time
from elevenlabs.conversational_ai.conversation import ClientTools


class TestClientTools:
    """Test suite for ClientTools functionality."""
    
    def test_default_initialization(self):
        """Test that ClientTools can be initialized without parameters (backwards compatibility)."""
        client_tools = ClientTools()
        assert client_tools._custom_loop is None
        assert client_tools._loop is None
        assert not client_tools._running.is_set()
        
    def test_custom_loop_initialization(self):
        """Test that ClientTools can be initialized with a custom event loop."""
        loop = asyncio.new_event_loop()
        try:
            client_tools = ClientTools(loop=loop)
            assert client_tools._custom_loop is loop
            assert client_tools._loop is None
            assert not client_tools._running.is_set()
        finally:
            loop.close()
    
    def test_start_with_default_loop(self):
        """Test starting ClientTools with default behavior (creates own loop)."""
        client_tools = ClientTools()
        
        try:
            client_tools.start()
            assert client_tools._running.is_set()
            assert client_tools._loop is not None
            assert client_tools._thread is not None
            assert client_tools._thread.is_alive()
        finally:
            client_tools.stop()
    
    @pytest.mark.asyncio
    async def test_start_with_custom_loop(self):
        """Test starting ClientTools with a custom event loop."""
        custom_loop = asyncio.get_running_loop()
        client_tools = ClientTools(loop=custom_loop)
        
        try:
            client_tools.start()
            assert client_tools._running.is_set()
            assert client_tools._loop is custom_loop
            assert client_tools._thread is None  # No thread created for custom loop
        finally:
            client_tools.stop()
    
    def test_tool_registration(self):
        """Test that tools can be registered correctly."""
        client_tools = ClientTools()
        
        def sync_tool(params):
            return "sync result"
        
        async def async_tool(params):
            return "async result"
        
        client_tools.register("sync_tool", sync_tool, is_async=False)
        client_tools.register("async_tool", async_tool, is_async=True)
        
        assert "sync_tool" in client_tools.tools
        assert "async_tool" in client_tools.tools
        
        sync_handler, sync_is_async = client_tools.tools["sync_tool"]
        async_handler, async_is_async = client_tools.tools["async_tool"]
        
        assert sync_handler is sync_tool
        assert not sync_is_async
        assert async_handler is async_tool
        assert async_is_async
    
    def test_duplicate_tool_registration(self):
        """Test that registering a tool with the same name raises an error."""
        client_tools = ClientTools()
        
        def tool(params):
            return "result"
        
        client_tools.register("test_tool", tool)
        
        with pytest.raises(ValueError, match="Tool 'test_tool' is already registered"):
            client_tools.register("test_tool", tool)
    
    def test_invalid_handler_registration(self):
        """Test that registering a non-callable handler raises an error."""
        client_tools = ClientTools()
        
        with pytest.raises(ValueError, match="Handler must be callable"):
            client_tools.register("invalid_tool", "not_callable")
    
    @pytest.mark.asyncio
    async def test_sync_tool_execution_with_custom_loop(self):
        """Test executing a sync tool with a custom event loop."""
        custom_loop = asyncio.get_running_loop()
        client_tools = ClientTools(loop=custom_loop)
        
        def sync_tool(params):
            return f"sync result: {params.get('input', 'default')}"
        
        client_tools.register("sync_tool", sync_tool, is_async=False)
        client_tools.start()
        
        try:
            result = await client_tools.handle("sync_tool", {"input": "test"})
            assert result == "sync result: test"
        finally:
            client_tools.stop()
    
    @pytest.mark.asyncio
    async def test_async_tool_execution_with_custom_loop(self):
        """Test executing an async tool with a custom event loop."""
        custom_loop = asyncio.get_running_loop()
        client_tools = ClientTools(loop=custom_loop)
        
        async def async_tool(params):
            await asyncio.sleep(0.01)  # Simulate async work
            return f"async result: {params.get('input', 'default')}"
        
        client_tools.register("async_tool", async_tool, is_async=True)
        client_tools.start()
        
        try:
            result = await client_tools.handle("async_tool", {"input": "test"})
            assert result == "async result: test"
        finally:
            client_tools.stop()
    
    @pytest.mark.asyncio
    async def test_tool_execution_with_callback_custom_loop(self):
        """Test executing a tool via callback mechanism with custom event loop."""
        custom_loop = asyncio.get_running_loop()
        client_tools = ClientTools(loop=custom_loop)
        
        def sync_tool(params):
            return f"callback result: {params.get('data', 'none')}"
        
        client_tools.register("callback_tool", sync_tool, is_async=False)
        client_tools.start()
        
        callback_results = []
        
        def test_callback(response):
            callback_results.append(response)
        
        try:
            client_tools.execute_tool(
                "callback_tool", 
                {"tool_call_id": "test123", "data": "callback_test"}, 
                test_callback
            )
            
            # Wait for callback to be executed
            await asyncio.sleep(0.1)
            
            assert len(callback_results) == 1
            response = callback_results[0]
            assert response["type"] == "client_tool_result"
            assert response["tool_call_id"] == "test123"
            assert "callback result: callback_test" in response["result"]
            assert response["is_error"] is False
        finally:
            client_tools.stop()
    
    def test_sync_tool_execution_with_default_loop(self):
        """Test executing a sync tool with default loop behavior."""
        client_tools = ClientTools()
        
        def sync_tool(params):
            return f"default sync: {params.get('value', 'empty')}"
        
        client_tools.register("sync_tool", sync_tool, is_async=False)
        client_tools.start()
        
        # Use asyncio.run to test from a fresh event loop
        async def test():
            result = await client_tools.handle("sync_tool", {"value": "default_test"})
            return result
        
        try:
            result = asyncio.run(test())
            assert result == "default sync: default_test"
        finally:
            client_tools.stop()
    
    @pytest.mark.asyncio
    async def test_unregistered_tool_error(self):
        """Test that calling an unregistered tool raises an error."""
        custom_loop = asyncio.get_running_loop()
        client_tools = ClientTools(loop=custom_loop)
        client_tools.start()
        
        try:
            with pytest.raises(ValueError, match="Tool 'nonexistent' is not registered"):
                await client_tools.handle("nonexistent", {})
        finally:
            client_tools.stop()
    
    def test_execute_tool_without_start(self):
        """Test that execute_tool raises error when not started."""
        client_tools = ClientTools()
        
        def callback(response):
            pass
        
        with pytest.raises(RuntimeError, match="ClientTools event loop is not running"):
            client_tools.execute_tool("any_tool", {}, callback)
    
    @pytest.mark.asyncio
    async def test_tool_exception_handling(self):
        """Test that tool exceptions are handled correctly."""
        custom_loop = asyncio.get_running_loop()
        client_tools = ClientTools(loop=custom_loop)
        
        def failing_tool(params):
            raise ValueError("Tool failed intentionally")
        
        client_tools.register("failing_tool", failing_tool, is_async=False)
        client_tools.start()
        
        callback_results = []
        
        def error_callback(response):
            callback_results.append(response)
        
        try:
            client_tools.execute_tool(
                "failing_tool",
                {"tool_call_id": "error_test"},
                error_callback
            )
            
            # Wait for callback
            await asyncio.sleep(0.1)
            
            assert len(callback_results) == 1
            response = callback_results[0]
            assert response["type"] == "client_tool_result"
            assert response["tool_call_id"] == "error_test"
            assert response["is_error"] is True
            assert "Tool failed intentionally" in response["result"]
        finally:
            client_tools.stop()
    
    @pytest.mark.asyncio
    async def test_event_loop_isolation(self):
        """Test that custom loop prevents 'different event loop' errors."""
        # This test simulates the scenario mentioned in the GitHub issue
        main_loop = asyncio.get_running_loop()
        
        # Create ClientTools with the current loop
        client_tools = ClientTools(loop=main_loop)
        
        async def loop_aware_tool(params):
            # This tool checks which loop it's running on
            current_loop = asyncio.get_running_loop()
            return f"Running on loop: {id(current_loop)}"
        
        client_tools.register("loop_tool", loop_aware_tool, is_async=True)
        client_tools.start()
        
        try:
            result = await client_tools.handle("loop_tool", {})
            # The result should indicate it's running on the same loop
            expected_loop_id = str(id(main_loop))
            assert expected_loop_id in result
        finally:
            client_tools.stop()
    
    def test_stop_custom_loop_behavior(self):
        """Test that stop() doesn't close custom loops."""
        custom_loop = asyncio.new_event_loop()
        client_tools = ClientTools(loop=custom_loop)
        
        try:
            client_tools.start()
            assert client_tools._running.is_set()
            
            client_tools.stop()
            assert not client_tools._running.is_set()
            
            # Custom loop should still be open and usable
            assert not custom_loop.is_closed()
        finally:
            custom_loop.close()
    
    def test_stop_default_loop_behavior(self):
        """Test that stop() properly cleans up default loops."""
        client_tools = ClientTools()
        
        client_tools.start()
        assert client_tools._running.is_set()
        assert client_tools._thread is not None
        thread = client_tools._thread
        
        client_tools.stop()
        assert not client_tools._running.is_set()
        
        # Wait for thread to finish
        thread.join(timeout=1.0)
        assert not thread.is_alive()